package com.platform.yijia.controller;

import com.google.gson.Gson;
import com.platform.yijia.pojo.*;
import com.platform.yijia.service.*;
import com.platform.yijia.utils.CodeMsg;
import com.platform.yijia.utils.ResultData;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Random;

/*
 * <Title> IntegralShoppingMallController </Title>
 * <Description> 油站积分商城控制层 </Description>
 * @Author JK
 * @Date 2021年2月24日
 */
@Controller
@RequestMapping("/api")
public class IntegralShoppingMallController {
    private static Logger logger =(Logger) LoggerFactory.getLogger(IntegralShoppingMallController.class);
    @Resource
    private IntegralWaresService integralWaresService;
    @Resource
    private IntegralOrderService integralOrderService;
    @Resource
    private CustomerPointsService customerPointsService;
    @Resource
    private StationService stationService;
    @Resource
    private CustomerPointsRecordService customerPointsRecordService;

    //获取油站积分商品信息
    @RequestMapping(value = "/getIntegralWaresInfoList", method = RequestMethod.GET)
    @ResponseBody
    public String getIntegralWaresInfoList(@RequestParam Integer stationId){
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        if(stationId !=null){
            IntegralWares integralWares = new IntegralWares();
            integralWares.setStationId(stationId);
            List<IntegralWares> integralWaresInfoList = integralWaresService.getIntegralWaresInfoList(integralWares);
            resultData=ResultData.success(integralWaresInfoList);
        }else{
            resultData=ResultData.error(CodeMsg.REQUEST_FAIL);
        }
        return gson.toJson(resultData);
    }

    //生成积分订单
    @RequestMapping(value = "/insertIntegralOrder", method = RequestMethod.POST)
    @ResponseBody
    @Transactional
    public String insertIntegralOrder(@RequestBody IntegralOrder request){
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        IntegralOrder integralOrder = new IntegralOrder();      //积分订单
        IntegralWares integralWares = new IntegralWares();      //积分商品表
        CustomerPoints customerPoints = new CustomerPoints();       //客户积分表
        CustomerPointsRecord customerPointsRecord = new CustomerPointsRecord();     //客户积分记录

        if(StringUtils.isNotBlank(request.getCustomerName())){
            integralOrder.setCustomerName(request.getCustomerName());
            customerPointsRecord.setCustomerName(request.getCustomerName());
        }
        if(StringUtils.isNotBlank(request.getWaresName())){
            integralOrder.setWaresName(request.getWaresName());
        }
        if(request.getExchangeNum() != null){
            integralOrder.setExchangeNum(request.getExchangeNum());
        }
        if(request.getStationId() != null){
            integralOrder.setStationId(request.getStationId());
            customerPointsRecord.setStationId(request.getStationId());
        }
        if(request.getWaresId() !=null){
            integralOrder.setWaresId(request.getWaresId());
            integralWares.setId(request.getWaresId());
        }
        if(StringUtils.isNotBlank(request.getUnionId())){
            integralOrder.setUnionId(request.getUnionId());
            customerPoints.setUnionId(request.getUnionId());
            customerPointsRecord.setUnionId(request.getUnionId());
        }
        if(request.getIntegral() !=null){
            integralOrder.setIntegral(request.getIntegral());
            customerPointsRecord.setIntegral(request.getIntegral());
        }

        integralOrder.setStatus("1");   //订单状态

        //订单规则 时间+6位随机数
        Random random = new Random();
        String str="";
        for(int i=0; i<6; i++){
            str+=random.nextInt(10);
        }
        integralOrder.setIntegralOrderNo(System.nanoTime()+str);
        integralOrder.setExchangeTime(new Date());

        //更新用户积分
        CustomerPoints customerPointsInfo = customerPointsService.getCustomerPointsInfo(customerPoints);
        if(customerPointsInfo !=null){
            //用户剩余积分
            BigDecimal points = new BigDecimal(customerPointsInfo.getPoints()).subtract(new BigDecimal(request.getIntegral()));
            customerPoints.setPoints(Integer.valueOf(points.toString()));
            //用户已消费积分累积
            BigDecimal consumptionPoints = new BigDecimal(customerPointsInfo.getConsumptionPoints()).add(new BigDecimal(request.getIntegral()));
            customerPoints.setConsumptionPoints(Integer.valueOf(consumptionPoints.toString()));
            customerPointsService.updateCustomerPointsInfo(customerPoints);
        }

        //油站名称
        Map<String, String> m = stationService.getStationAppIdAndAppSecret(request.getStationId());
        if(m !=null && m.containsKey("stationName")){
            integralOrder.setStationName(m.get("stationName"));
            customerPointsRecord.setStationName(m.get("stationName"));
        }

        List<IntegralWares> integralWaresInfoList = integralWaresService.getIntegralWaresInfoList(integralWares);
        if(integralWaresInfoList !=null && integralWaresInfoList.size() ==1){
            //更新商品已售数量
            integralWares.setWaresOutCount(integralWaresInfoList.get(0).getWaresOutCount()+1);
            integralWares.setWaresCount(integralWaresInfoList.get(0).getWaresCount()-1);
            integralWaresService.updateIntegralWaresInfo(integralWares);
        }

        //插入客户记录表
        customerPointsRecord.setRecordType("-");    //客户积分表记录
        customerPointsRecord.setCreateTime(new Date());
        customerPointsRecordService.insertCustomerPointsInfo(customerPointsRecord);


        boolean b = integralOrderService.insertIntegralOrder(integralOrder);
        if (b){
            resultData=ResultData.success(CodeMsg.SUCCESS);
        }else {
            resultData=ResultData.success(CodeMsg.REQUEST_FAIL);
        }
        return gson.toJson(resultData);
    }

    //获取用户积分订单列表
    @RequestMapping(value = "/getUserIntegralOrderList", method = RequestMethod.GET)
    @ResponseBody
    public String getUserIntegralOrderList(@RequestParam String unionId){
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        if(unionId !=null){
            IntegralOrder integralOrder = new IntegralOrder();
            integralOrder.setUnionId(unionId);
            //integralOrder.setStationId(stationId);
            List<IntegralOrder> integralOrderList = integralOrderService.getUserIntegralOrderList(integralOrder);
            resultData=ResultData.success(integralOrderList);
        }else {
            resultData=ResultData.success(CodeMsg.REQUEST_FAIL);
        }
        return gson.toJson(resultData);
    }

    //获取客户积分信息
    @RequestMapping(value = "/getCustomerPointsInfo", method = RequestMethod.GET)
    @ResponseBody
    public String getCustomerPointsInfo(@RequestParam("unionId") String unionId){
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        if(unionId !=null){
            CustomerPoints customerPoints  =new CustomerPoints();
            customerPoints.setUnionId(unionId);
            //customerPoints.setStationId(stationId);
            CustomerPoints customerPointsInfo = customerPointsService.getCustomerPointsInfo(customerPoints);
            resultData=ResultData.success(customerPointsInfo);
        }else {
            resultData=ResultData.success(CodeMsg.REQUEST_FAIL);
        }
        return gson.toJson(resultData);
    }

    //获取客户积分记录信息
    @RequestMapping(value = "/getCustomerPointsRecordList", method = RequestMethod.GET)
    @ResponseBody
    public String getCustomerPointsRecordList(@RequestParam("unionId") String unionId){
        Gson gson =new Gson();
        //返回结果集
        ResultData resultData = null;
        if(unionId !=null){
            CustomerPointsRecord customerPointsRecord  =new CustomerPointsRecord();
            customerPointsRecord.setUnionId(unionId);
            //customerPointsRecord.setStationId(stationId);
            List<CustomerPointsRecord> customerPointsRecordList = customerPointsRecordService.getCustomerPointsRecordList(customerPointsRecord);
            resultData=ResultData.success(customerPointsRecordList);
        }else {
            resultData=ResultData.success(CodeMsg.REQUEST_FAIL);
        }
        return gson.toJson(resultData);
    }
}
